@@ -41,6 +41,7 @@ struct _EvDocumentPrivate
 	gchar          *uri;
 	guint64         file_size;
 
+	gboolean        cache_loaded;
 	gint            n_pages;
 
 	gboolean        uniform;
@@ -199,8 +200,7 @@ ev_document_setup_cache (EvDocument *document)
         /* Cache some info about the document to avoid
          * going to the backends since it requires locks
          */
-	priv->info = _ev_document_get_info (document);
-        priv->n_pages = _ev_document_get_n_pages (document);
+	priv->cache_loaded = TRUE;
 
         for (i = 0; i < priv->n_pages; i++) {
                 EvPage     *page = ev_document_get_page (document, i);
@@ -294,9 +294,10 @@ ev_document_initialize_synctex (EvDocument  *document,
 }
 
 /**
- * ev_document_load:
+ * ev_document_load_full:
  * @document: a #EvDocument
  * @uri: the document's URI
+ * @flags: flags from #EvDocumentLoadFlags
  * @error: a #GError location to store an error, or %NULL
  *
  * Loads @document from @uri.
@@ -311,9 +312,10 @@ ev_document_initialize_synctex (EvDocument  *document,
  * Returns: %TRUE on success, or %FALSE on failure.
  */
 gboolean
-ev_document_load (EvDocument  *document,
-		  const char  *uri,
-		  GError     **error)
+ev_document_load_full (EvDocument           *document,
+		       const char           *uri,
+		       EvDocumentLoadFlags   flags,
+		       GError              **error)
 {
 	EvDocumentClass *klass = EV_DOCUMENT_GET_CLASS (document);
 	gboolean retval;
@@ -334,7 +336,10 @@ ev_document_load (EvDocument  *document,
 					     "Internal error in backend");
 		}
 	} else {
-                ev_document_setup_cache (document);
+		document->priv->info = _ev_document_get_info (document);
+		document->priv->n_pages = _ev_document_get_n_pages (document);
+		if (!(flags & EV_DOCUMENT_LOAD_FLAG_NO_CACHE))
+			ev_document_setup_cache (document);
 		document->priv->uri = g_strdup (uri);
 		document->priv->file_size = _ev_document_get_size (uri);
 		ev_document_initialize_synctex (document, uri);
@@ -343,6 +348,32 @@ ev_document_load (EvDocument  *document,
 	return retval;
 }
 
+/**
+ * ev_document_load:
+ * @document: a #EvDocument
+ * @uri: the document's URI
+ * @error: a #GError location to store an error, or %NULL
+ *
+ * Loads @document from @uri.
+ *
+ * On failure, %FALSE is returned and @error is filled in.
+ * If the document is encrypted, EV_DEFINE_ERROR_ENCRYPTED is returned.
+ * If the backend cannot load the specific document, EV_DOCUMENT_ERROR_INVALID
+ * is returned. Other errors are possible too, depending on the backend
+ * used to load the document and the URI, e.g. #GIOError, #GFileError, and
+ * #GConvertError.
+ *
+ * Returns: %TRUE on success, or %FALSE on failure.
+ */
+gboolean
+ev_document_load (EvDocument  *document,
+		  const char  *uri,
+		  GError     **error)
+{
+	return ev_document_load_full (document, uri,
+				      EV_DOCUMENT_LOAD_FLAG_NONE, error);
+}
+
 /**
  * ev_document_load_stream:
  * @document: a #EvDocument
@@ -382,7 +413,11 @@ ev_document_load_stream (EvDocument         *document,
         if (!klass->load_stream (document, stream, flags, cancellable, error))
                 return FALSE;
 
-        ev_document_setup_cache (document);
+	document->priv->info = _ev_document_get_info (document);
+	document->priv->n_pages = _ev_document_get_n_pages (document);
+
+        if (!(flags & EV_DOCUMENT_LOAD_FLAG_NO_CACHE))
+                ev_document_setup_cache (document);
 
         return TRUE;
 }
@@ -426,7 +461,12 @@ ev_document_load_gfile (EvDocument         *document,
         if (!klass->load_gfile (document, file, flags, cancellable, error))
                 return FALSE;
 
-        ev_document_setup_cache (document);
+	document->priv->info = _ev_document_get_info (document);
+	document->priv->n_pages = _ev_document_get_n_pages (document);
+
+        if (!(flags & EV_DOCUMENT_LOAD_FLAG_NO_CACHE))
+                ev_document_setup_cache (document);
+
 	document->priv->uri = g_file_get_uri (file);
 	document->priv->file_size = _ev_document_get_size_gfile (file);
 	ev_document_initialize_synctex (document, document->priv->uri);
@@ -649,17 +689,31 @@ ev_document_get_page_size (EvDocument *document,
 			   double     *width,
 			   double     *height)
 {
+	EvDocumentPrivate *priv;
+
 	g_return_if_fail (EV_IS_DOCUMENT (document));
 	g_return_if_fail (page_index >= 0 || page_index < document->priv->n_pages);
 
-	if (width)
-		*width = document->priv->uniform ?
-			document->priv->uniform_width :
-			document->priv->page_sizes[page_index].width;
-	if (height)
-		*height = document->priv->uniform ?
-			document->priv->uniform_height :
-			document->priv->page_sizes[page_index].height;
+	priv = document->priv;
+
+	if (priv->cache_loaded) {
+		if (width)
+			*width = priv->uniform ?
+				priv->uniform_width :
+				priv->page_sizes[page_index].width;
+		if (height)
+			*height = priv->uniform ?
+				priv->uniform_height :
+				priv->page_sizes[page_index].height;
+	} else {
+		EvPage *page;
+
+		g_mutex_lock (&ev_doc_mutex);
+		page = ev_document_get_page (document, page_index);
+		_ev_document_get_page_size (document, page, width, height);
+		g_object_unref (page);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
 }
 
 static gchar *
@@ -679,6 +733,19 @@ ev_document_get_page_label (EvDocument *document,
 	g_return_val_if_fail (EV_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (page_index >= 0 || page_index < document->priv->n_pages, NULL);
 
+	if (!document->priv->cache_loaded) {
+		EvPage *page;
+		gchar *page_label;
+
+		g_mutex_lock (&ev_doc_mutex);
+		page = ev_document_get_page (document, page_index);
+		page_label = _ev_document_get_page_label (document, page);
+		g_object_unref (page);
+		g_mutex_unlock (&ev_doc_mutex);
+
+		return page_label ? page_label : g_strdup_printf ("%d", page_index + 1);
+	}
+
 	return (document->priv->page_labels && document->priv->page_labels[page_index]) ?
 		g_strdup (document->priv->page_labels[page_index]) :
 		g_strdup_printf ("%d", page_index + 1);
@@ -802,6 +869,12 @@ ev_document_is_page_size_uniform (EvDocument *document)
 {
 	g_return_val_if_fail (EV_IS_DOCUMENT (document), TRUE);
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	return document->priv->uniform;
 }
 
@@ -812,6 +885,12 @@ ev_document_get_max_page_size (EvDocument *document,
 {
 	g_return_if_fail (EV_IS_DOCUMENT (document));
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	if (width)
 		*width = document->priv->max_width;
 	if (height)
@@ -825,6 +904,12 @@ ev_document_get_min_page_size (EvDocument *document,
 {
 	g_return_if_fail (EV_IS_DOCUMENT (document));
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	if (width)
 		*width = document->priv->min_width;
 	if (height)
@@ -836,6 +921,12 @@ ev_document_check_dimensions (EvDocument *document)
 {
 	g_return_val_if_fail (EV_IS_DOCUMENT (document), FALSE);
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	return (document->priv->max_width > 0 && document->priv->max_height > 0);
 }
 
@@ -852,6 +943,12 @@ ev_document_get_max_label_len (EvDocument *document)
 {
 	g_return_val_if_fail (EV_IS_DOCUMENT (document), -1);
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	return document->priv->max_label;
 }
 
@@ -860,6 +957,12 @@ ev_document_has_text_page_labels (EvDocument *document)
 {
 	g_return_val_if_fail (EV_IS_DOCUMENT (document), FALSE);
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
 	return document->priv->page_labels != NULL;
 }
 
@@ -877,6 +980,12 @@ ev_document_find_page_by_label (EvDocument  *document,
 	g_return_val_if_fail (page_label != NULL, FALSE);
 	g_return_val_if_fail (page_index != NULL, FALSE);
 
+	if (!document->priv->cache_loaded) {
+		g_mutex_lock (&ev_doc_mutex);
+		ev_document_setup_cache (document);
+		g_mutex_unlock (&ev_doc_mutex);
+	}
+
         /* First, look for a literal label match */
 	for (i = 0; priv->page_labels && i < priv->n_pages; i ++) {
 		if (priv->page_labels[i] != NULL &&
