@@ -852,6 +852,11 @@ ev_job_thumbnail_dispose (GObject *object)
 		job->thumbnail = NULL;
 	}
 
+	if (job->thumbnail_surface) {
+		cairo_surface_destroy (job->thumbnail_surface);
+		job->thumbnail_surface = NULL;
+	}
+
 	(* G_OBJECT_CLASS (ev_job_thumbnail_parent_class)->dispose) (object);
 }
 
@@ -1569,8 +1574,10 @@ ev_job_save_run (EvJob *job)
 	ev_tmp_uri_unlink (local_uri);
 
         /* Copy the metadata from the original file */
-        if (!error)
-                ev_file_copy_metadata (job_save->document_uri, job_save->uri, &error);
+        if (!error) {
+                /* Ignore errors here. Failure to copy metadata is not a hard error */
+                ev_file_copy_metadata (job_save->document_uri, job_save->uri, NULL);
+        }
 
 	if (error) {
 		ev_job_failed_from_error (job, error);
